@@ -12,6 +12,7 @@ use ibc::{
         ics02_client::misbehaviour::MisbehaviourEvidence,
         ics03_connection::connection::{ConnectionEnd, IdentifiedConnectionEnd},
         ics03_connection::version::Version,
+        ics04_channel,
         ics04_channel::channel::{ChannelEnd, IdentifiedChannelEnd},
         ics04_channel::packet::{PacketMsgType, Sequence},
         ics23_commitment::commitment::CommitmentPrefix,
@@ -37,8 +38,8 @@ use ibc_proto::ibc::core::connection::v1::QueryClientConnectionsRequest;
 use ibc_proto::ibc::core::connection::v1::QueryConnectionsRequest;
 
 use crate::{
-    chain::StatusResponse, config::ChainConfig, connection::ConnectionMsgType, error::Error,
-    keyring::KeyEntry,
+    chain::handle::requests::AppVersion, chain::StatusResponse, config::ChainConfig,
+    connection::ConnectionMsgType, error::Error, keyring::KeyEntry,
 };
 
 use super::{reply_channel, ChainHandle, ChainRequest, HealthCheck, ReplyTo, Subscription};
@@ -135,11 +136,8 @@ impl ChainHandle for ProdChainHandle {
         })
     }
 
-    fn module_version(&self, port_id: &PortId) -> Result<String, Error> {
-        self.send(|reply_to| ChainRequest::ModuleVersion {
-            port_id: port_id.clone(),
-            reply_to,
-        })
+    fn app_version(&self, request: AppVersion) -> Result<ics04_channel::Version, Error> {
+        self.send(|reply_to| ChainRequest::AppVersion { request, reply_to })
     }
 
     fn query_status(&self) -> Result<StatusResponse, Error> {
