@@ -19,6 +19,7 @@ use ibc::{
             version::Version,
         },
         ics04_channel::{
+            self,
             channel::{ChannelEnd, IdentifiedChannelEnd},
             packet::{PacketMsgType, Sequence},
         },
@@ -44,6 +45,7 @@ use ibc_proto::ibc::core::{
 };
 
 use crate::{
+    chain::handle::requests::AppVersion,
     chain::StatusResponse,
     config::ChainConfig,
     connection::ConnectionMsgType,
@@ -66,6 +68,64 @@ pub struct Threads {
     pub event_monitor: Option<thread::JoinHandle<()>>,
 }
 
+#[derive(Debug)]
+pub enum EventMonitorCtrl {
+    None {
+        /// Empty channel for when the None case
+        never: EventReceiver,
+    },
+    Live {
+        /// Receiver channel from the event bus
+        event_receiver: EventReceiver,
+
+        /// Sender channel to terminate the event monitor
+        tx_monitor_cmd: TxMonitorCmd,
+    },
+}
+
+impl EventMonitorCtrl {
+    pub fn none() -> Self {
+        Self::None {
+            never: channel::never(),
+        }
+    }
+
+    pub fn live(event_receiver: EventReceiver, tx_monitor_cmd: TxMonitorCmd) -> Self {
+        Self::Live {
+            event_receiver,
+            tx_monitor_cmd,
+        }
+    }
+
+    pub fn enable(&mut self, event_receiver: EventReceiver, tx_monitor_cmd: TxMonitorCmd) {
+        *self = Self::live(event_receiver, tx_monitor_cmd);
+    }
+
+    pub fn recv(&self) -> &EventReceiver {
+        match self {
+            Self::None { ref never } => never,
+            Self::Live {
+                ref event_receiver, ..
+            } => event_receiver,
+        }
+    }
+
+    pub fn shutdown(&self) -> Result<(), Error> {
+        match self {
+            Self::None { .. } => Ok(()),
+            Self::Live {
+                ref tx_monitor_cmd, ..
+            } => tx_monitor_cmd
+                .send(MonitorCmd::Shutdown)
+                .map_err(Error::send),
+        }
+    }
+
+    pub fn is_live(&self) -> bool {
+        matches!(self, Self::Live { .. })
+    }
+}
+
 pub struct ChainRuntime<Endpoint: ChainEndpoint> {
     /// The specific chain this runtime runs against
     chain: Endpoint,
@@ -81,11 +141,8 @@ pub struct ChainRuntime<Endpoint: ChainEndpoint> {
     /// An event bus, for broadcasting events that this runtime receives (via `event_receiver`) to subscribers
     event_bus: EventBus<Arc<MonitorResult<EventBatch>>>,
 
-    /// Receiver channel from the event bus
-    event_receiver: EventReceiver,
-
-    /// Sender channel to terminate the event monitor
-    tx_monitor_cmd: TxMonitorCmd,
+    /// Interface to the event monitor
+    event_monitor_ctrl: EventMonitorCtrl,
 
     /// A handle to the light client
     light_client: Endpoint::LightClient,
@@ -109,11 +166,8 @@ where
         // Start the light client
         let light_client = chain.init_light_client()?;
 
-        // Start the event monitor
-        let (event_batch_rx, tx_monitor_cmd) = chain.init_event_monitor(rt.clone())?;
-
         // Instantiate & spawn the runtime
-        let (handle, _) = Self::init(chain, light_client, event_batch_rx, tx_monitor_cmd, rt);
+        let (handle, _) = Self::init(chain, light_client, rt);
 
         Ok(handle)
     }
@@ -122,11 +176,9 @@ where
     fn init<Handle: ChainHandle>(
         chain: Endpoint,
         light_client: Endpoint::LightClient,
-        event_receiver: EventReceiver,
-        tx_monitor_cmd: TxMonitorCmd,
         rt: Arc<TokioRuntime>,
     ) -> (Handle, thread::JoinHandle<()>) {
-        let chain_runtime = Self::new(chain, light_client, event_receiver, tx_monitor_cmd, rt);
+        let chain_runtime = Self::new(chain, light_client, rt);
 
         // Get a handle to the runtime
         let handle: Handle = chain_runtime.handle();
@@ -143,13 +195,7 @@ where
     }
 
     /// Basic constructor
-    fn new(
-        chain: Endpoint,
-        light_client: Endpoint::LightClient,
-        event_receiver: EventReceiver,
-        tx_monitor_cmd: TxMonitorCmd,
-        rt: Arc<TokioRuntime>,
-    ) -> Self {
+    fn new(chain: Endpoint, light_client: Endpoint::LightClient, rt: Arc<TokioRuntime>) -> Self {
         let (request_sender, request_receiver) = channel::unbounded::<ChainRequest>();
 
         Self {
@@ -158,8 +204,7 @@ where
             request_sender,
             request_receiver,
             event_bus: EventBus::new(),
-            event_receiver,
-            tx_monitor_cmd,
+            event_monitor_ctrl: EventMonitorCtrl::none(),
             light_client,
         }
     }
@@ -174,7 +219,7 @@ where
     fn run(mut self) -> Result<(), Error> {
         loop {
             channel::select! {
-                recv(self.event_receiver) -> event_batch => {
+                recv(self.event_monitor_ctrl.recv()) -> event_batch => {
                     match event_batch {
                         Ok(event_batch) => {
                             self.event_bus
@@ -189,8 +234,7 @@ where
                 recv(self.request_receiver) -> event => {
                     match event {
                         Ok(ChainRequest::Shutdown { reply_to }) => {
-                            self.tx_monitor_cmd.send(MonitorCmd::Shutdown)
-                                .map_err(Error::send)?;
+                            self.event_monitor_ctrl.shutdown()?;
 
                             let res = self.chain.shutdown();
                             reply_to.send(res)
@@ -227,13 +271,14 @@ where
                             self.get_key(reply_to)?
                         }
 
+                        Ok(ChainRequest::AppVersion { request, reply_to }) => {
+                            self.app_version(request, reply_to)?
+                        }
+
                         Ok(ChainRequest::AddKey { key_name, key, reply_to }) => {
                             self.add_key(key_name, key, reply_to)?
                         }
 
-                        Ok(ChainRequest::ModuleVersion { port_id, reply_to }) => {
-                            self.module_version(port_id, reply_to)?
-                        }
 
                         Ok(ChainRequest::BuildHeader { trusted_height, target_height, client_state, reply_to }) => {
                             self.build_header(trusted_height, target_height, client_state, reply_to)?
@@ -382,10 +427,23 @@ where
     }
 
     fn subscribe(&mut self, reply_to: ReplyTo<Subscription>) -> Result<(), Error> {
+        if !self.event_monitor_ctrl.is_live() {
+            self.enable_event_monitor()?;
+        }
+
         let subscription = self.event_bus.subscribe();
         reply_to.send(Ok(subscription)).map_err(Error::send)
     }
 
+    fn enable_event_monitor(&mut self) -> Result<(), Error> {
+        let (event_receiver, tx_monitor_cmd) = self.chain.init_event_monitor(self.rt.clone())?;
+
+        self.event_monitor_ctrl
+            .enable(event_receiver, tx_monitor_cmd);
+
+        Ok(())
+    }
+
     fn send_messages_and_wait_commit(
         &mut self,
         proto_msgs: Vec<prost_types::Any>,
@@ -424,6 +482,15 @@ where
         reply_to.send(result).map_err(Error::send)
     }
 
+    fn app_version(
+        &self,
+        request: AppVersion,
+        reply_to: ReplyTo<ics04_channel::Version>,
+    ) -> Result<(), Error> {
+        let result = self.chain.query_app_version(request);
+        reply_to.send(result).map_err(Error::send)
+    }
+
     fn add_key(
         &mut self,
         key_name: String,
@@ -434,11 +501,6 @@ where
         reply_to.send(result).map_err(Error::send)
     }
 
-    fn module_version(&self, port_id: PortId, reply_to: ReplyTo<String>) -> Result<(), Error> {
-        let result = self.chain.query_module_version(&port_id);
-        reply_to.send(Ok(result)).map_err(Error::send)
-    }
-
     fn build_header(
         &mut self,
         trusted_height: Height,
