@@ -1,5 +1,3 @@
-use std::convert::{TryFrom, TryInto};
-
 use itertools::Itertools;
 
 use tendermint_light_client::{
@@ -14,19 +12,21 @@ use tendermint_light_client::{
 use tendermint_rpc as rpc;
 
 use ibc::{
-    downcast,
-    ics02_client::{
-        client_state::AnyClientState,
-        client_type::ClientType,
-        events::UpdateClient,
-        header::{AnyHeader, Header},
-        misbehaviour::{Misbehaviour, MisbehaviourEvidence},
-    },
-    ics07_tendermint::{
+    clients::ics07_tendermint::{
         header::{headers_compatible, Header as TmHeader},
         misbehaviour::Misbehaviour as TmMisbehaviour,
     },
-    ics24_host::identifier::ChainId,
+    core::{
+        ics02_client::{
+            client_state::AnyClientState,
+            client_type::ClientType,
+            events::UpdateClient,
+            header::{AnyHeader, Header},
+            misbehaviour::{Misbehaviour, MisbehaviourEvidence},
+        },
+        ics24_host::identifier::ChainId,
+    },
+    downcast,
 };
 use tracing::trace;
 
@@ -124,7 +124,7 @@ impl super::LightClient<CosmosSdkChain> for LightClient {
             ibc::Height::new(self.chain_id.version(), latest_chain_block.height().into());
 
         // set the target height to the minimum between the update height and latest chain height
-        let target_height = std::cmp::min(update.consensus_height(), latest_chain_height);
+        let target_height = core::cmp::min(update.consensus_height(), latest_chain_height);
         let trusted_height = update_header.trusted_height;
 
         // TODO - check that a consensus state at trusted_height still exists on-chain,
